﻿using UnityEngine;
using System.Collections;


/// <summary>
/// Extension class
/// </summary>
public static class Extension
{
	//Some informations usable in the future:
	//
	//Instead of messaging system in Unity, use delagets (Listener)
	//http://www.codeproject.com/Articles/11541/The-Simplest-C-Events-Example-Imaginable

	

	//========================================================================================
	//Extensions
	//========================================================================================

	//========================================================================================
	// Quaternion
	//========================================================================================


	public static float Length(this Quaternion q)
	{
		return Mathf.Sqrt(q.x * q.x + q.y * q.y + q.z * q.z + q.w * q.w);
	}

	public static Quaternion Normalize(this Quaternion q)
	{

		float length = 1.0f / q.Length();
		q.x *= length;
		q.y *= length;
		q.z *= length;
		q.w *= length;

		return q;
	}

	//========================================================================================
	// Vector4
	//========================================================================================


	/// <summary>
	/// Transform Vector4 by Matrix
	/// It is the same as vector = matrix * vector
	/// </summary>
	/// <param name="v">vector to be transformed</param>
	/// <param name="m">transformation matrix</param>
	/// <returns>transformed vector</returns>
	public static Vector4 Transform(this Vector4 v, Matrix4x4 m)
	{
		Vector4 result;
			
		result.x = (v.x * m.m00) + (v.y * m.m01) + (v.z * m.m02) + (v.w * m.m03);
		result.y = (v.x * m.m10) + (v.y * m.m11) + (v.z * m.m12) + (v.w * m.m13);
		result.z = (v.x * m.m20) + (v.y * m.m21) + (v.z * m.m22) + (v.w * m.m23);
		result.w = (v.x * m.m30) + (v.y * m.m31) + (v.z * m.m32) + (v.w * m.m33);
		
		return result;
	}

	//========================================================================================
	// Vector3
	//========================================================================================

	
	/// <summary>
	/// Test if vector is Infinity
	/// All component are Negative or Positive Infinity
	/// </summary>
	/// <param name="target"></param>
	/// <returns></returns>
	public static bool IsInfinity(this Vector3 target) 
	{
		if (float.IsInfinity(target.x)) return true;
		if (float.IsInfinity(target.y)) return true;
		if (float.IsInfinity(target.z)) return true;

		return false;
	}

	/// <summary>
	/// Get max value in vector coordinates
	/// </summary>
	/// <param name="v"></param>
	/// <returns></returns>
	public static float MaxValue(this Vector3 v) 
	{
		float m = v.x;
		if (v.y > m) m = v.y;
		if (v.z > m) m = v.z;
		return m;
	}

	/// <summary>
	/// Get min value in vector coordinates
	/// </summary>
	/// <param name="v"></param>
	/// <returns></returns>
	public static float MinValue(this Vector3 v)
	{
		float m = v.x;
		if (v.y < m) m = v.y;
		if (v.z < m) m = v.z;
		return m;
	}

	/// <summary>
	/// Get middle value in vector coordinates
	/// </summary>
	/// <param name="v"></param>
	/// <returns></returns>
	public static float MiddleValue(this Vector3 v)
	{		
		return Mathf.Max(Mathf.Min(v.x, v.y), Mathf.Min(Mathf.Max(v.x, v.y), v.z));
	}

	/// <summary>
	/// Squared distance to the point p
	/// </summary>
	/// <param name="target"></param>
	/// <param name="p">second point</param>
	/// <returns></returns>
	public static float DistanceSquared(this Vector3 target, Vector3 p)
	{
		return (target - p).sqrMagnitude;		
	}

	//========================================================================================
	// GameObject
	//========================================================================================


	/// <summary>
	/// Get child component of game object by its name
	/// </summary>
	/// <typeparam name="T"></typeparam>
	/// <param name="target"></param>
	/// <param name="name"></param>
	/// <returns></returns>
	public static T GetChildComponenentByName<T>(this GameObject target, string name) where T : Component
	{
		return GetChildComponenentByName<T>(target.transform, name);
	}


	//========================================================================================
	// Component
	//========================================================================================


	/// <summary>
	/// Extension for Component
	/// Get children by its name and return component of type T
	/// </summary>
	/// <param name="target"></param>
	/// <param name="name"></param>
	/// <returns></returns>
	public static T GetChildComponenentByName<T>(this Component target, string name) where T : Component
	{		
		return GetChildComponenentByName<T>(target.transform, name);
	}


	//========================================================================================
	// Transform
	//========================================================================================


	/// <summary>
	/// Extension for Transform
	/// Get children by its name and return component of type T
	/// Every GameObject has Transform Component, it is used to find child
	/// </summary>
	/// <param name="target"></param>
	/// <param name="name"></param>
	/// <returns></returns>
	public static T GetChildComponenentByName<T>(this Transform target, string name) where T : Component
	{		
		if (target.name == name)
		{			
			return target.GetComponent<T>();
		}

		for (int i = 0; i < target.childCount; ++i)
		{
			Component result = GetChildComponenentByName<T>(target.GetChild(i), name);

			if (result != null)
			{
				
				return result.GetComponent<T>();
			}
		}

		return null;

	}


	//========================================================================================
	// RenderTexture
	//========================================================================================


	/// <summary>
	/// Save RenderTexture to file
	/// </summary>
	/// <param name="rtt"></param>
	/// <param name="fileName"></param>
	public static void SaveToFile(this RenderTexture rtt, string fileName) 
	{
		RenderTexture.active = rtt;
		Texture2D virtualPhoto = new Texture2D(rtt.width, rtt.height, TextureFormat.RGB24, false); // false, meaning no need for mipmaps		
		virtualPhoto.ReadPixels(new Rect(0, 0, rtt.width, rtt.height), 0, 0);
		RenderTexture.active = null;

		Texture2D.Destroy(virtualPhoto);

		byte[] bytes;
		bytes = virtualPhoto.EncodeToPNG();

		System.IO.File.WriteAllBytes(fileName, bytes);
	}
}
