﻿using UnityEngine;

public static class Utils
{

	//========================================================================================
	//Helper structures
	//========================================================================================

	public struct AxisSystem
	{
		public Vector3 forward;
		public Vector3 up;
		public Vector3 right;

		public Quaternion GetRotation() 
		{
			Quaternion q = Quaternion.identity;
			q.SetLookRotation(forward, up);
			return q;
		}

	};


	//========================================================================================
	//Helper methods
	//========================================================================================

	/// <summary>
	/// Swap two values passed by ref
	/// </summary>
	/// <typeparam name="T"></typeparam>
	/// <param name="lhs"></param>
	/// <param name="rhs"></param>
	public static void Swap<T>(ref T lhs, ref T rhs)
	{
		T temp = lhs;
		lhs = rhs;
		rhs = temp;
	}


	/// <summary>
	/// Calculate random point inside AABB
	/// </summary>
	/// <param name="box"></param>
	/// <returns></returns>
	public static Vector3 RandomInBox(BoxCollider box)
	{
		return RandomInBox(box.bounds.min, box.bounds.max);
	}

	/// <summary>
	/// Calculate random point inside AABB
	/// </summary>
	/// <param name="min"></param>
	/// <param name="max"></param>
	/// <returns></returns>
	public static Vector3 RandomInBox(Vector3 min, Vector3 max)
	{
		Vector3 v = new Vector3();
		v.x = Random.Range(min.x, max.x);
		v.y = Random.Range(min.y, max.y);
		v.z = Random.Range(min.z, max.z);

		return v;
	}

	public static int RandomIndexWithProbability(float[] probability) 
	{
		float x = Random.value;
		float sum = 0;
		int val = 0;
		for (int i = 0; i < probability.Length; i++)
		{
			sum += probability[i];
			if (sum > x)
			{
				val = i;
				break;
			}
		}

		return val;
	}

	/// <summary>
	/// Map value s from interval [fromStart, fromEnd] to interval [toStart, toEnd]
	/// </summary>
	/// <param name="fromStart"></param>
	/// <param name="fromEnd"></param>
	/// <param name="toStart"></param>
	/// <param name="toEnd"></param>
	/// <param name="s"></param>
	/// <returns></returns>
	public static float MapRange(
		float fromStart, float fromEnd, 
		float toStart, float toEnd, 
		float s)
	{
		return toStart + (s - fromStart) * (toEnd - toStart) / (fromEnd - fromStart);
	}


	public static Vector3 Lerp(Vector3 from, Vector3 to, float t) 
	{
		Vector3 v;
		v.x = Mathf.Lerp(from.x, to.x, t);
		v.y = Mathf.Lerp(from.y, to.y, t);
		v.z = Mathf.Lerp(from.z, to.z, t);

		return v;
	}


    public static Quaternion Lerp(Quaternion from, Quaternion to, float t)
    {
        Quaternion v;
        v.x = Mathf.Lerp(from.x, to.x, t);
        v.y = Mathf.Lerp(from.y, to.y, t);
        v.z = Mathf.Lerp(from.z, to.z, t);
        v.w = Mathf.Lerp(from.w, to.w, t);

        return v;
    }


	public static bool RaySphereIntersection(Ray ray, Vector3 center, float radius)
	{
		Vector3 Q = center - ray.origin;
		double c = Q.sqrMagnitude;
		double v = Vector3.Dot(Q, ray.direction);
		double d = radius * radius - (c - v * v);

		// If there was no intersection, return -1
		if (d < 0.0)
		{
			return false;
		}

		return true;
		
	}

}
