﻿using UnityEngine;
using System.Collections;

/// <summary>
/// View frustum helper class
/// </summary>
public class ViewFrustum
{
	public enum Face { FRONT = 1, BACK = 2 };
    public enum Space { WORLD = 1, VIEW = 2 };

	private Camera cam;

	public ViewFrustum(Camera cam) 
	{
		this.cam = cam;
	}
	

	/// <summary>
	/// Get frustum corners in World Space
	/// 
	///		G ------------ H
	///	   / |			/|
	///	  /	 |		   / |
	///  /	 | E -----/--| F
	///  C --/------ D  /
	///  |  /		 | /
	///  | /		 |/
	///  A --------- B
	///  	
	/// Face, that we want can be specified 
	/// FRONT - ABCD
	/// BACK - EFGH
	/// FRONT | BACK - ABCDEFGH
	/// </summary>
	/// <param name="corners">array with corners to be filled (if size is not enough, array is reallocated)</param>
	/// <param name="face">face we want (default: FRONT | BACK - all corners)</param>
	/// <param name="scale">offsetting corners (default: 1.0 - no offset)</param>
	public void GetCorners(ref Vector3[] corners, Face face = Face.FRONT | Face.BACK, Space space = Space.WORLD, float scale = 1.0f)
	{
		
		//DX - LH
		/*
		Vector4[] cornersVector4 = 
		{
			//front
			new Vector4(-1, -1, 0, 1), //A
			new Vector4( 1, -1, 0, 1), //B
			new Vector4( 1,  1, 0, 1), //C		
			new Vector4(-1,  1, 0, 1), //D

			//back
			new Vector4(-1, -1, 1, 1), //E
			new Vector4( 1, -1, 1, 1), //F
			new Vector4( 1,  1, 1, 1), //G		
			new Vector4(-1,  1, 1, 1), //H
		};
		*/

		//GL - RH
		/*
		MyMath::Vector4 cornersVector4[] = 
		{
			//front
			MyMath::Vector4(-1, -1, -1, 1), //A
			MyMath::Vector4( 1, -1, -1, 1), //B
			MyMath::Vector4( 1,  1, -1, 1), //C		
			MyMath::Vector4(-1,  1, -1, 1), //D

			//back
			MyMath::Vector4(-1, -1, 1, 1), //E
			MyMath::Vector4( 1, -1, 1, 1), //F
			MyMath::Vector4( 1,  1, 1, 1), //G		
			MyMath::Vector4(-1,  1, 1, 1), //H
		};
		*/
		
		//LH system (same as in GL)
		
		Vector4[] cornersVector4 = 
		{
			//front
			new Vector4(-1, -1, 0, 1), //A
			new Vector4( 1, -1, 0, 1), //B
            new Vector4(-1,  1, 0, 1), //C
			new Vector4( 1,  1, 0, 1), //D					
	
			//back
			new Vector4(-1, -1, 1, 1), //E
			new Vector4( 1, -1, 1, 1), //F
            new Vector4(-1,  1, 1, 1), //H
			new Vector4( 1,  1, 1, 1), //H							
		};
		
		int start = 0;
		int end = 8;
		if (face == Face.FRONT)
		{
			start = 0;
			end = 4;
		}
		else if (face == Face.BACK)
		{
			start = 4;
			end = 8;		
		}

		if (corners.Length != end - start)
		{
			corners = new Vector3[end - start];
		}

		Matrix4x4 inv;

        if (space == Space.WORLD)
        {
            inv = Matrix4x4.Inverse(this.cam.projectionMatrix * this.cam.worldToCameraMatrix);
        }
        else 
        {
            inv = Matrix4x4.Inverse(this.cam.projectionMatrix);
        }
		
		float invW = 0;
		int index = 0;
		for (int i = start; i < end; i++)
		{

            //cornersVector4[i] = cornersVector4[i].Transform(inv);
            cornersVector4[i] = inv * cornersVector4[i];

			invW = 1.0f / cornersVector4[i].w;
						
			corners[index].x = cornersVector4[i].x * invW;
			corners[index].y = cornersVector4[i].y * invW;
			corners[index].z = cornersVector4[i].z * invW;
			index++;
		}

		if (scale != 1.0f)
		{
			Vector3 center = new Vector3(0, 0, 0);
			for (int i = 0; i < corners.Length; i++)
			{
				center += corners[i];
			}
			center *= 1.0f / corners.Length;
			
			for (int i = 0; i < corners.Length; i++)
			{
				corners[i] += (corners[i] - center) * (scale - 1.0f);
			}
		}
	
	}

}
